
// ----------------------------------------------------------------------
// 
// savedspl.c
//
// Saves and restores personal display mode settings
//
// By Aldo Giove - (c) 2002
//
// ----------------------------------------------------------------------

#include <windows.h>
#include <windowsx.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "resource.h"

// ----------------------------------------------------------------------

#define UWM_ICONNOTIFY WM_USER + 1

HINSTANCE hInstance;
HMENU hMenu;
HWND hWndMain = NULL;
int selectedLanguage = ID_ENGLISH;
char appName[80];

// ----------------------------------------------------------------------

// Forward declarations
void showMessage(const char *msg);

// ----------------------------------------------------------------------

// Loads a resource string, in the currently selected language
void loadResourceString(int id, LPSTR s, int maxLen)
{
    int id1 = id + selectedLanguage;
    int retval = LoadString(hInstance, id1, s, maxLen);
    if (!retval)
    {
        TCHAR msg[120];
        sprintf(msg, "Error loading resource string #%d.", id1);
        showMessage(msg);
        s[0] = '\0';
    }
}

// ----------------------------------------------------------------------

// Returns a pointer to a resource string
LPCSTR getResourceString(int id)
{
    static TCHAR s[2048];
    loadResourceString(id, s, sizeof(s));
    return s;
}

// ----------------------------------------------------------------------

// Shows an alert box
void showMessage(const char *msg)
{
    MessageBox(NULL, msg, appName, MB_ICONHAND|MB_APPLMODAL);
}

// ----------------------------------------------------------------------

void saveKeyValue(const char *name, const char *value)
{
    LONG retval;
    HKEY hk;
    DWORD dw;
    
    retval = RegCreateKeyEx(HKEY_CURRENT_USER,
                            "Software\\l-d-x.com\\SaveDspl",
                            0,
                            "REG_SZ",
                            REG_OPTION_NON_VOLATILE,
                            KEY_ALL_ACCESS,
                            NULL,
                            &hk,
                            &dw);
    if (retval == ERROR_SUCCESS)
    {
        retval = RegSetValueEx(hk, name, 0, REG_SZ, value, strlen(value)+1);
        RegCloseKey(hk);
    }
    
    if (retval != ERROR_SUCCESS)
        showMessage(getResourceString(IDS_FAILEDSAVEKEY));
}

// ----------------------------------------------------------------------

BOOL readKeyValue(const char *name, char *value, DWORD maxLen)
{
    LONG retval;
    HKEY hk;
    
    retval = RegOpenKeyEx(HKEY_CURRENT_USER,
                          "Software\\l-d-x.com\\SaveDspl",
                          0,
                          KEY_READ,
                          &hk);

    if (retval == ERROR_SUCCESS)
    {
        DWORD type;
        retval = RegQueryValueEx(hk,
                                 name,
                                 0,
                                 &type,
                                 value,
                                 &maxLen);
        RegCloseKey(hk);
    }
    
    if (retval == ERROR_SUCCESS)
        return TRUE;
    return FALSE;
}

// ----------------------------------------------------------------------

// Selects a display mode in the format "WIDTHxHEIGHTxDEPTHxFREQUENCY"
BOOL selectDisplayMode(const char *mode)
{
    // Scan the string to read width, height, depth and frequency

    char s[256];

    DWORD width = 0,
          height = 0,
          depth = 0,
          frequency = 0;

    char *token;
    
    strcpy(s, mode);
    s[sizeof(s)-2] = '\0';
    strcat(s, "x"); // For the last "strtok" call

    token = strtok(s, "x");
    if (token)
    {
        width = atoi(token);
        
        token = strtok(NULL, "x");
        if (token)
        {
            height = atoi(token);

            token = strtok(NULL, "x");
            if (token)
            {
                depth = atoi(token);

                token = strtok(NULL, "x");
                if (token)
                {
                    frequency = atoi(token);
                }
            }
        }
    }

    if (width && height && depth && frequency)
    {
        DEVMODE dm;
        int i = 0;
        BOOL ok = EnumDisplaySettings(NULL, i, &dm);
        BOOL found = FALSE;

        while (ok && !found)
        {
            if (dm.dmPelsWidth == width &&
                dm.dmPelsHeight == height &&
                dm.dmBitsPerPel == depth &&
                dm.dmDisplayFrequency == frequency)
            {
                found = TRUE;
            }

            if (!found)
                ok = EnumDisplaySettings(NULL, ++i, &dm);
        }

        if (found)
        {
            LONG retval = ChangeDisplaySettings(&dm, CDS_UPDATEREGISTRY);        
            if (retval == DISP_CHANGE_SUCCESSFUL)
                return TRUE;
        }
    }

    return FALSE;
}

// ----------------------------------------------------------------------

void saveDisplayMode(void)
{
    BOOL ok;
    DEVMODE dm;

    ok = EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dm);
    if (ok)
    {
        char s[80];

        sprintf(s, "%ldx%ldx%ldx%ld", 
                dm.dmPelsWidth, dm.dmPelsHeight,
                dm.dmBitsPerPel, dm.dmDisplayFrequency);

        // Saves settings in the current user's registry area
        saveKeyValue("DisplaySettings", s);
    }
    else
        showMessage(getResourceString(IDS_FAILEDGETMODE));
}

// ----------------------------------------------------------------------

int openDocument(const char *name, int nCmdShow)
{
	HWND hwnd = NULL;
	HINSTANCE h;
    h = ShellExecute(hwnd, NULL, name, NULL, NULL, nCmdShow);

	if ((long)h <= 32)
        {
            char s[1500];

/*
            SE_ERR_ASSOCINCOMPLETE The filename association is incomplete or invalid.
            SE_ERR_DDEBUSY The DDE transaction could not be completed because other DDE                            transactions were being processed.
            SE_ERR_DDEFAIL The DDE transaction failed.
            SE_ERR_DDETIMEOUT The DDE transaction could not be completed because the
                              request timed out.
            SE_ERR_NOASSOC There is no application associated with the given filename                            extension.
            SE_ERR_SHARE A sharing violation occurred.

            0 The system is out of memory or resources.
            ERROR_FILE_NOT_FOUND The specified file was not found.
            ERROR_PATH_NOT_FOUND The specified path was not found.
            ERROR_BAD_FORMAT The .EXE file is invalid (non-Win32 .EXE or error in .EXE                              image).
*/

        switch ((long)h)
        {
            default:
                sprintf(s, "Errore %ld nell'apertura del documento '%s'.\r\n\r\n"
                           "Error %ld while opening document '%s'.",
                       (long)h, name,
                       (long)h, name);
        }

        showMessage(s);
        return 2;
    }
    return 0;
}

// ----------------------------------------------------------------------

void updateTrayIcon(HWND hWnd)
{
    static HICON hIcon = NULL;
    NOTIFYICONDATA nid;
    DWORD msg = NIM_ADD;

    hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_SAVEDSP2));

    memset(&nid, 0, sizeof(nid));

    nid.cbSize = sizeof(nid);
    nid.hWnd = hWnd;
    nid.uID = 1;
    nid.uFlags = NIF_ICON|NIF_MESSAGE|NIF_TIP;
    nid.uCallbackMessage = UWM_ICONNOTIFY;
    nid.hIcon = hIcon;
    loadResourceString(IDS_ICONTIP, nid.szTip, sizeof(nid.szTip));

    Shell_NotifyIcon(msg, &nid);
}

// ----------------------------------------------------------------------

void cleanup(HWND hWnd)
{
    NOTIFYICONDATA nid;

    KillTimer(hWnd, 1);

    // Removes icon from tray area
    memset(&nid, 0, sizeof(nid));
    nid.cbSize = sizeof(nid);
    nid.hWnd = hWnd;
    nid.uID = 1;
    Shell_NotifyIcon(NIM_DELETE, &nid);
}

// ----------------------------------------------------------------------

// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            {
                HWND hEdit1 = GetDlgItem(hDlg, IDC_EDIT1);
                Edit_SetText(hEdit1, getResourceString(IDS_INSTRUCTIONS));
                return TRUE;
            }
            break;
        
        case WM_COMMAND:
            switch (LOWORD(wParam))
            {
                case IDOK:
                case IDCANCEL:
                    EndDialog(hDlg, LOWORD(wParam));
                    return TRUE;
                    break;
                case IDC_LDX_COM:
                    openDocument("http://www.l-d-x.com/", SW_SHOW);
                    break;
            }
            break;
    }
    return FALSE;
}

// ----------------------------------------------------------------------

// Main window procedure
LRESULT CALLBACK wndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message) 
    {
        case WM_CREATE:
            break;

        case WM_TIMER:
            updateTrayIcon(hWnd);
            break;

        case UWM_ICONNOTIFY:
            {
                // Message from tray icon
                switch (lParam)
                {
                    case WM_RBUTTONUP:
                        {
                            //HMENU hMenu = GetMenu(hWnd);
                            HMENU hPopup = GetSubMenu(hMenu, 0);

			                // Get the current cursor position, to display the menu at
			                POINT mouse;
			                GetCursorPos(&mouse);

                            SetMenuDefaultItem(hPopup, 0, TRUE);
			                
			                // There's a "bug"
			                // (Microsoft calls it a feature) in Windows 95 that requires calling
			                // SetForegroundWindow. To find out more, search for Q135788 in MSDN.
			                //
			                SetForegroundWindow(hWnd);

			                // Display the menu at the desired position
			                TrackPopupMenu(hPopup, TPM_LEFTBUTTON|TPM_RIGHTBUTTON, 
                                           mouse.x, mouse.y, 0, hWnd, NULL);
                            break;
                        }

                    case WM_LBUTTONDBLCLK:
                        PostMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDM_ABOUT, 0), 0);
                        SetForegroundWindow(hWnd);
                        break;
                }
                break;
            }

        case WM_COMMAND:
            {
                WORD wmId    = LOWORD(wParam),
                     wmEvent = HIWORD(wParam); 

                switch (wmId)
                {
                    case IDM_EXIT:
                        DestroyWindow(hWnd);
                        break;

                    // Message from tray icon
                    case IDM_ABOUT:
                          DialogBox(hInstance, (LPCTSTR)IDD_ABOUTBOX, 
                                    hWnd, (DLGPROC)About);

                    default:
                        return DefWindowProc(hWnd, message, wParam, lParam);
                }
                break;
            }

        case WM_PAINT:
            break;

        case WM_DESTROY:
            cleanup(hWnd);
            PostQuitMessage(0);
            break;

        case WM_ENDSESSION:
            // Display settings are saved only if session is ending
            if (wParam)
                saveDisplayMode();
            break;

        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
    }
    return 0;
}

// ----------------------------------------------------------------------

ATOM myRegisterClass(HINSTANCE hInstance)
{
    WNDCLASSEX wcex;

    loadResourceString(IDS_APP_TITLE, appName, sizeof(appName));

    memset(&wcex, 0, sizeof(wcex));
    
    wcex.cbSize = sizeof(WNDCLASSEX); 
    
    wcex.style			= CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc	= (WNDPROC)wndProc;
    wcex.cbClsExtra		= 0;
    wcex.cbWndExtra		= 0;
    wcex.hInstance		= hInstance;
    wcex.lpszClassName	= appName;
    
    return RegisterClassEx(&wcex);
}

// ----------------------------------------------------------------------

void initInstance(HINSTANCE hInst)
{
    HWND hWnd;

    hInstance = hInst;
 
    hWnd = CreateWindow(appName, appName, WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, 
                        NULL, NULL, hInstance, NULL);

    // Starts timer
    SetTimer(hWnd, 1, 2000, NULL);

    // Loads menu
    hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(IDM_SAVEDSPL+selectedLanguage));

    hWndMain = hWnd;
}

// ----------------------------------------------------------------------

int APIENTRY WinMain(HINSTANCE hInst,
                     HINSTANCE hPrevInst,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
    char s[512];
    BOOL ok = TRUE;
    MSG msg;

    LANGID langId = GetSystemDefaultLangID();

    if (langId == 0x0410 || langId == 0x0810)
        selectedLanguage = ID_ITALIANO;

    myRegisterClass(hInst);
    initInstance(hInst);

    s[0] = '\0';

    // Command-line overrides registry-saved value
    if (*lpCmdLine)
        ok = selectDisplayMode(lpCmdLine);	
    else
    {
        if (readKeyValue("DisplaySettings", s, sizeof(s)+1))
            ok = selectDisplayMode(s);
    }

    if (!ok)
    {
        loadResourceString(IDS_FAILEDVIDEOMODE, s, sizeof(s));
        if (*lpCmdLine)
            strcat(s, lpCmdLine);
        else
            strcat(s, "(null)");

        showMessage(s);
    }

    // If display settings from command line, do not stay resident
    // in the tray area and terminate now
    if (*lpCmdLine)
        DestroyWindow(hWndMain);

    // Main message loop:
    while (GetMessage(&msg, NULL, 0, 0)) 
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    
    return msg.wParam;
}

// ----------------------------------------------------------------------
